@@ -136,7 +136,7 @@ do { \
 #define FMT_PREP(_fmt) \
 do { \
 	_fmt##len = strlen(format) + 1; \
-	_fmt##copy = kmalloc(_fmt##len, GFP_KERNEL); \
+	_fmt##copy = kmalloc(_fmt##len, irql_gfp()); \
 	if (_fmt##copy) { \
 		memcpy(_fmt##copy, format, _fmt##len); \
 		strip_l_modifier(_fmt##copy); \
@@ -158,6 +158,7 @@ do { \
 
 #endif /* !CONFIG_X86_64 */
 
+__attribute__((format(printf, 2, 3)))
 noregparm INT WIN_FUNC(_win_sprintf,12)
 	(char *buf, const char *format, ...)
 {
@@ -200,6 +201,7 @@ noregparm INT WIN_FUNC(_win_vsprintf,3)
 	EXIT2(return i);
 }
 
+__attribute__((format(printf, 3, 4)))
 noregparm INT WIN_FUNC(_win_snprintf,12)
 	(char *buf, SIZE_T count, const char *format, ...)
 {
@@ -217,6 +219,7 @@ noregparm INT WIN_FUNC(_win_snprintf,12)
 	return res;
 }
 
+__attribute__((format(printf, 3, 4)))
 noregparm INT WIN_FUNC(_win__snprintf,12)
 	(char *buf, SIZE_T count, const char *format, ...)
 {
@@ -270,6 +273,19 @@ noregparm INT WIN_FUNC(_win__vsnprintf,4)
 	EXIT2(return i);
 }
 
+noregparm INT WIN_FUNC(_win__vsnwprintf,4)
+	(wchar_t *str, SIZE_T size, const wchar_t *format, va_list ap)
+{
+	int ret;
+
+	TODO();		/* format expansion not implemented */
+	_win_wcsncpy(str, format, size);
+	ret = _win_wcslen(format);
+	if (ret >= size)
+		ret = -1;
+	return ret;
+}
+
 noregparm char *WIN_FUNC(_win_strncpy,3)
 	(char *dst, char *src, SIZE_T n)
 {
@@ -354,7 +370,7 @@ noregparm wchar_t *WIN_FUNC(_win_wcscpy,2)
 {
 	wchar_t *d = dest;
 	while ((*d++ = *src++))
-	       ;
+		;
 	return dest;
 }
 
@@ -461,7 +477,7 @@ noregparm int WIN_FUNC(rand,0)
 	int i, n;
 
 	get_random_bytes(buf, sizeof(buf));
-	for (n = i = 0; i < sizeof(buf) ; i++)
+	for (n = i = 0; i < sizeof(buf); i++)
 		n += buf[i];
 	return n;
 }
@@ -473,6 +489,12 @@ noregparm int WIN_FUNC(_win_atoi,1)
 	return i;
 }
 
+noregparm int WIN_FUNC(_win_isdigit,1)
+	(int c)
+{
+	return isdigit(c);
+}
+
 noregparm int WIN_FUNC(_win_isprint,1)
 	(int c)
 {
@@ -482,61 +504,61 @@ noregparm int WIN_FUNC(_win_isprint,1)
 wstdcall s64 WIN_FUNC(_alldiv,2)
 	(s64 a, s64 b)
 {
-	return (a / b);
+	return a / b;
 }
 
 wstdcall u64 WIN_FUNC(_aulldiv,2)
 	(u64 a, u64 b)
 {
-	return (a / b);
+	return a / b;
 }
 
 wstdcall s64 WIN_FUNC(_allmul,2)
 	(s64 a, s64 b)
 {
-	return (a * b);
+	return a * b;
 }
 
 wstdcall u64 WIN_FUNC(_aullmul,2)
 	(u64 a, u64 b)
 {
-	return (a * b);
+	return a * b;
 }
 
 wstdcall s64 WIN_FUNC(_allrem,2)
 	(s64 a, s64 b)
 {
-	return (a % b);
+	return a % b;
 }
 
 wstdcall u64 WIN_FUNC(_aullrem,2)
 	(u64 a, u64 b)
 {
-	return (a % b);
+	return a % b;
 }
 
-__attribute__((regparm(3))) s64 WIN_FUNC(_allshl,2)
+regparm3 s64 WIN_FUNC(_allshl,2)
 	(s64 a, u8 b)
 {
-	return (a << b);
+	return a << b;
 }
 
-__attribute__((regparm(3))) u64 WIN_FUNC(_aullshl,2)
+regparm3 u64 WIN_FUNC(_aullshl,2)
 	(u64 a, u8 b)
 {
-	return (a << b);
+	return a << b;
 }
 
-__attribute__((regparm(3))) s64 WIN_FUNC(_allshr,2)
+regparm3 s64 WIN_FUNC(_allshr,2)
 	(s64 a, u8 b)
 {
-	return (a >> b);
+	return a >> b;
 }
 
-__attribute__((regparm(3))) u64 WIN_FUNC(_aullshr,2)
+regparm3 u64 WIN_FUNC(_aullshr,2)
 	(u64 a, u8 b)
 {
-	return (a >> b);
+	return a >> b;
 }
 
 int stricmp(const char *s1, const char *s2)
@@ -565,14 +587,3 @@ void dump_bytes(const char *ctx, const u8 *from, int len)
 	printk(KERN_DEBUG "%s: %p: %s\n", ctx, from, buf);
 	kfree(buf);
 }
-
-int crt_init(void)
-{
-	return 0;
-}
-
-/* called when module is being removed */
-void crt_exit(void)
-{
-	EXIT4(return);
-}
