@@ -43,8 +43,8 @@ spinlock_t ntoskernel_lock;
 static void *mdl_cache;
 static struct nt_list wrap_mdl_list;
 
-static work_struct_t kdpc_work;
-static void kdpc_worker(worker_param_t dummy);
+static struct work_struct kdpc_work;
+static void kdpc_worker(struct work_struct *dummy);
 
 static struct nt_list kdpc_list;
 static spinlock_t kdpc_list_lock;
@@ -61,14 +61,14 @@ struct bus_driver {
 
 static struct nt_list bus_driver_list;
 
-static work_struct_t ntos_work;
+static struct work_struct ntos_work;
 static struct nt_list ntos_work_list;
 static spinlock_t ntos_work_lock;
-static void ntos_work_worker(worker_param_t dummy);
-static struct nt_thread *ntos_worker_thread;
+static void ntos_work_worker(struct work_struct *dummy);
 spinlock_t irp_cancel_lock;
 static NT_SPIN_LOCK nt_list_lock;
 static struct nt_slist wrap_timer_slist;
+CCHAR cpu_count;
 
 /* compute ticks (100ns) since 1601 until when system booted into
  * wrap_ticks_to_boot */
@@ -81,20 +81,20 @@ static void update_user_shared_data_proc(unsigned long data);
 #endif
 
 WIN_SYMBOL_MAP("KeTickCount", &jiffies)
-
+WIN_SYMBOL_MAP("KeNumberProcessors", &cpu_count)
 WIN_SYMBOL_MAP("NlsMbCodePageTag", FALSE)
 
-workqueue_struct_t *ntos_wq;
+struct workqueue_struct *ntos_wq;
 
 #ifdef WRAP_PREEMPT
-DEFINE_PER_CPU(irql_info_t, irql_info);
+DEFINE_PER_CPU(struct irql_info, irql_info);
 #endif
 
 #if defined(CONFIG_X86_64)
 static void update_user_shared_data_proc(unsigned long data)
 {
 	/* timer is supposed to be scheduled every 10ms, but bigger
-	 * intervals seem to work (tried upto 50ms) */
+	 * intervals seem to work (tried up to 50ms) */
 	*((ULONG64 *)&kuser_shared_data.system_time) = ticks_1601();
 	*((ULONG64 *)&kuser_shared_data.interrupt_time) =
 		jiffies * TICKSPERSEC / HZ;
@@ -143,7 +143,7 @@ void *allocate_object(ULONG size, enum common_object_type type,
 	return body;
 }
 
-void free_object(void *object)
+static void free_object(void *object)
 {
 	struct common_object_header *hdr;
 
@@ -444,15 +444,14 @@ void wrap_init_timer(struct nt_timer *nt_timer, enum timer_type type,
 	 * freed, so we use slack_kmalloc so it gets freed when driver
 	 * is unloaded */
 	if (nmb)
-		wrap_timer = kmalloc(sizeof(*wrap_timer), irql_gfp());
+		wrap_timer = kzalloc(sizeof(*wrap_timer), irql_gfp());
 	else
-		wrap_timer = slack_kmalloc(sizeof(*wrap_timer));
+		wrap_timer = slack_kzalloc(sizeof(*wrap_timer));
 	if (!wrap_timer) {
 		ERROR("couldn't allocate memory for timer");
 		return;
 	}
 
-	memset(wrap_timer, 0, sizeof(*wrap_timer));
 	init_timer(&wrap_timer->timer);
 	wrap_timer->timer.data = (unsigned long)wrap_timer;
 	wrap_timer->timer.function = timer_proc;
@@ -462,7 +461,7 @@ void wrap_init_timer(struct nt_timer *nt_timer, enum timer_type type,
 #endif
 	nt_timer->wrap_timer = wrap_timer;
 	nt_timer->kdpc = NULL;
-	initialize_object(&nt_timer->dh, type, 0);
+	initialize_object(&nt_timer->dh, (enum dh_type)type, 0);
 	nt_timer->wrap_timer_magic = WRAP_TIMER_MAGIC;
 	TIMERTRACE("timer %p (%p)", wrap_timer, nt_timer);
 	spin_lock_bh(&ntoskernel_lock);
@@ -532,7 +531,7 @@ wstdcall BOOLEAN WIN_FUNC(KeSetTimerEx,4)
 {
 	unsigned long expires_hz, repeat_hz;
 
-	TIMERENTER("%p, %Ld, %d", nt_timer, duetime_ticks, period_ms);
+	TIMERENTER("%p, %lld, %d", nt_timer, duetime_ticks, period_ms);
 	expires_hz = SYSTEM_TIME_TO_HZ(duetime_ticks);
 	repeat_hz = MSEC_TO_HZ(period_ms);
 	return wrap_set_timer(nt_timer, expires_hz, repeat_hz, kdpc);
@@ -542,7 +541,7 @@ wstdcall BOOLEAN WIN_FUNC(KeSetTimer,3)
 	(struct nt_timer *nt_timer, LARGE_INTEGER duetime_ticks,
 	 struct kdpc *kdpc)
 {
-	TIMERENTER("%p, %Ld, %p", nt_timer, duetime_ticks, kdpc);
+	TIMERENTER("%p, %lld, %p", nt_timer, duetime_ticks, kdpc);
 	return KeSetTimerEx(nt_timer, duetime_ticks, 0, kdpc);
 }
 
@@ -588,11 +587,11 @@ wstdcall void WIN_FUNC(KeInitializeDpc,3)
 	ENTER3("%p, %p, %p", kdpc, func, ctx);
 	memset(kdpc, 0, sizeof(*kdpc));
 	kdpc->func = func;
-	kdpc->ctx  = ctx;
+	kdpc->ctx = ctx;
 	InitializeListHead(&kdpc->list);
 }
 
-static void kdpc_worker(worker_param_t dummy)
+static void kdpc_worker(struct work_struct *dummy)
 {
 	struct nt_list *entry;
 	struct kdpc *kdpc;
@@ -648,7 +647,7 @@ BOOLEAN queue_kdpc(struct kdpc *kdpc)
 	}
 	spin_unlock_irqrestore(&kdpc_list_lock, flags);
 	if (ret == TRUE)
-		schedule_ntos_work(&kdpc_work);
+		queue_work(ntos_wq, &kdpc_work);
 	WORKTRACE("%d", ret);
 	return ret;
 }
@@ -692,7 +691,7 @@ wstdcall void WIN_FUNC(KeSetImportanceDpc,2)
 	kdpc->importance = importance;
 }
 
-static void ntos_work_worker(worker_param_t dummy)
+static void ntos_work_worker(struct work_struct *dummy)
 {
 	struct ntos_work_item *ntos_work_item;
 	struct nt_list *cur;
@@ -730,7 +729,7 @@ int schedule_ntos_work_item(NTOS_WORK_FUNC func, void *arg1, void *arg2)
 	spin_lock_bh(&ntos_work_lock);
 	InsertTailList(&ntos_work_list, &ntos_work_item->list);
 	spin_unlock_bh(&ntos_work_lock);
-	schedule_ntos_work(&ntos_work);
+	queue_work(ntos_wq, &ntos_work);
 	WORKEXIT(return 0);
 }
 
@@ -796,14 +795,12 @@ wstdcall KIRQL WIN_FUNC(KeAcquireSpinLockRaiseToDpc,1)
 	return nt_spin_lock_irql(lock, DISPATCH_LEVEL);
 }
 
-#undef ExAllocatePoolWithTag
-
 wstdcall void *WIN_FUNC(ExAllocatePoolWithTag,3)
 	(enum pool_type pool_type, SIZE_T size, ULONG tag)
 {
 	void *addr;
 
-	ENTER4("pool_type: %d, size: %lu, tag: 0x%x", pool_type, size, tag);
+	ENTER4("pool_type: %d, size: %zu, tag: 0x%x", pool_type, size, tag);
 	assert_irql(_irql_ <= DISPATCH_LEVEL);
 	if (size < PAGE_SIZE)
 		addr = kmalloc(size, irql_gfp());
@@ -811,17 +808,17 @@ wstdcall void *WIN_FUNC(ExAllocatePoolWithTag,3)
 		if (irql_gfp() & GFP_ATOMIC) {
 			addr = __vmalloc(size, GFP_ATOMIC | __GFP_HIGHMEM,
 					 PAGE_KERNEL);
-			TRACE1("%p, %lu", addr, size);
+			TRACE1("%p, %zu", addr, size);
 		} else {
 			addr = vmalloc(size);
-			TRACE1("%p, %lu", addr, size);
+			TRACE1("%p, %zu", addr, size);
 		}
 	}
 	DBG_BLOCK(1) {
 		if (addr)
-			TRACE4("addr: %p, %lu", addr, size);
+			TRACE4("addr: %p, %zu", addr, size);
 		else
-			TRACE1("failed: %lu", size);
+			TRACE1("failed: %zu", size);
 	}
 	return addr;
 }
@@ -852,7 +849,7 @@ wstdcall void WIN_FUNC(ExInitializeNPagedLookasideList,7)
 	 LOOKASIDE_ALLOC_FUNC *alloc_func, LOOKASIDE_FREE_FUNC *free_func,
 	 ULONG flags, SIZE_T size, ULONG tag, USHORT depth)
 {
-	ENTER3("lookaside: %p, size: %lu, flags: %u, head: %p, "
+	ENTER3("lookaside: %p, size: %zu, flags: %u, head: %p, "
 	       "alloc: %p, free: %p", lookaside, size, flags,
 	       lookaside, alloc_func, free_func);
 
@@ -890,11 +887,6 @@ wstdcall void WIN_FUNC(ExDeleteNPagedLookasideList,1)
 	EXIT3(return);
 }
 
-#if defined(ALLOC_DEBUG) && ALLOC_DEBUG > 1
-#define ExAllocatePoolWithTag(pool_type, size, tag)			\
-	wrap_ExAllocatePoolWithTag(pool_type, size, tag, __FILE__, __LINE__)
-#endif
-
 wstdcall NTSTATUS WIN_FUNC(ExCreateCallback,4)
 	(struct callback_object **object, struct object_attributes *attributes,
 	 BOOLEAN create, BOOLEAN allow_multiple_callbacks)
@@ -1020,7 +1012,7 @@ static int grab_object(struct dispatcher_header *dh,
 }
 
 /* this function should be called holding dispatcher_lock */
-static void object_signalled(struct dispatcher_header *dh)
+static void object_signaled(struct dispatcher_header *dh)
 {
 	struct nt_list *cur, *next;
 	struct wait_block *wb;
@@ -1051,6 +1043,7 @@ wstdcall NTSTATUS WIN_FUNC(KeWaitForMultipleObjects,8)
 	typeof(jiffies) wait_hz = 0;
 	struct wait_block *wb, wb_array[THREAD_WAIT_OBJECTS];
 	struct dispatcher_header *dh;
+	KIRQL irql = current_irql();
 
 	EVENTENTER("%p, %d, %u, %p", current, count, wait_type, timeout);
 
@@ -1122,16 +1115,12 @@ wstdcall NTSTATUS WIN_FUNC(KeWaitForMultipleObjects,8)
 	else
 		wait_hz = SYSTEM_TIME_TO_HZ(*timeout);
 
-	DBG_BLOCK(2) {
-		KIRQL irql = current_irql();
-		if (irql >= DISPATCH_LEVEL) {
-			TRACE2("wait in atomic context: %lu, %d, %ld",
-			       wait_hz, in_atomic(), in_interrupt());
-		}
+	if (irql >= DISPATCH_LEVEL) {
+		WARNING("attempt to wait with irql %d", irql);
+		EVENTEXIT(return STATUS_INVALID_PARAMETER);
 	}
-	assert_irql(_irql_ < DISPATCH_LEVEL);
 	EVENTTRACE("%p: sleep for %ld on %p", current, wait_hz, &wait_done);
-	/* we don't honor 'alertable' - according to decription for
+	/* we don't honor 'alertable' - according to description for
 	 * this, even if waiting in non-alertable state, thread may be
 	 * alerted in some circumstances */
 	while (wait_count) {
@@ -1159,7 +1148,7 @@ wstdcall NTSTATUS WIN_FUNC(KeWaitForMultipleObjects,8)
 				EVENTEXIT(return STATUS_TIMEOUT);
 		}
 		assert(res > 0);
-		/* woken because object(s) signalled */
+		/* woken because object(s) signaled */
 		for (i = 0; wait_count && i < count; i++) {
 			if (!wb[i].thread || !wb[i].object)
 				continue;
@@ -1211,7 +1200,7 @@ wstdcall void WIN_FUNC(KeInitializeEvent,3)
 	(struct nt_event *nt_event, enum event_type type, BOOLEAN state)
 {
 	EVENTENTER("event = %p, type = %d, state = %d", nt_event, type, state);
-	initialize_object(&nt_event->dh, type, state);
+	initialize_object(&nt_event->dh, (enum dh_type)type, state);
 	EVENTEXIT(return);
 }
 
@@ -1227,7 +1216,7 @@ wstdcall LONG WIN_FUNC(KeSetEvent,3)
 	old_state = nt_event->dh.signal_state;
 	nt_event->dh.signal_state = 1;
 	if (old_state == 0)
-		object_signalled(&nt_event->dh);
+		object_signaled(&nt_event->dh);
 	spin_unlock_bh(&dispatcher_lock);
 	EVENTEXIT(return old_state);
 }
@@ -1290,7 +1279,7 @@ wstdcall LONG WIN_FUNC(KeReleaseMutex,2)
 		ret = mutex->dh.signal_state++;
 		if (ret == 0) {
 			mutex->owner_thread = NULL;
-			object_signalled(&mutex->dh);
+			object_signaled(&mutex->dh);
 		}
 	} else {
 		ret = STATUS_MUTANT_NOT_OWNED;
@@ -1308,8 +1297,8 @@ wstdcall void WIN_FUNC(KeInitializeSemaphore,3)
 {
 	EVENTENTER("%p: %d", semaphore, count);
 	/* if limit > 1, we need to satisfy as many waits (until count
-	 * becomes 0); so we keep decrementing count everytime a wait
-	 * is satisified */
+	 * becomes 0); so we keep decrementing count every time a wait
+	 * is satisfied */
 	initialize_object(&semaphore->dh, SemaphoreObject, count);
 	semaphore->dh.size = sizeof(*semaphore);
 	semaphore->limit = limit;
@@ -1334,7 +1323,7 @@ wstdcall LONG WIN_FUNC(KeReleaseSemaphore,4)
 		semaphore->dh.signal_state = semaphore->limit;
 	}
 	if (semaphore->dh.signal_state > 0)
-		object_signalled(&semaphore->dh);
+		object_signaled(&semaphore->dh);
 	spin_unlock_bh(&dispatcher_lock);
 	EVENTEXIT(return ret);
 }
@@ -1349,7 +1338,7 @@ wstdcall NTSTATUS WIN_FUNC(KeDelayExecutionThread,3)
 		ERROR("invalid wait_mode %d", wait_mode);
 
 	timeout = SYSTEM_TIME_TO_HZ(*interval);
-	EVENTTRACE("%p, %Ld, %ld", current, *interval, timeout);
+	EVENTTRACE("%p, %lld, %ld", current, *interval, timeout);
 	if (timeout <= 0)
 		EVENTEXIT(return STATUS_SUCCESS);
 
@@ -1382,7 +1371,7 @@ wstdcall void WIN_FUNC(KeQuerySystemTime,1)
 	(LARGE_INTEGER *time)
 {
 	*time = ticks_1601();
-	TRACE5("%Lu, %lu", *time, jiffies);
+	TRACE5("%llu, %lu", *time, jiffies);
 }
 
 wstdcall void WIN_FUNC(KeQueryTickCount,1)
@@ -1404,11 +1393,7 @@ wstdcall KAFFINITY WIN_FUNC(KeQueryActiveProcessors,0)
 {
 	int i, n;
 	KAFFINITY bits = 0;
-#ifdef num_online_cpus
 	n = num_online_cpus();
-#else
-	n = NR_CPUS;
-#endif
 	for (i = 0; i < n; i++)
 		bits = (bits << 1) | 1;
 	return bits;
@@ -1680,6 +1665,25 @@ wstdcall BOOLEAN WIN_FUNC(KeSynchronizeExecution,3)
 	return ret;
 }
 
+wstdcall BOOLEAN WIN_FUNC(KeRegisterBugCheckReasonCallback,4)
+	(void *callback_record, void *callback_routine, UINT reason,
+	 char *component)
+{
+	TRACE1("callback_record: %p, callback_routine: %p, reason: %d, "
+	       "component: %s", callback_record, callback_routine, reason,
+	       component);
+	TODO();
+	return FALSE;
+}
+
+wstdcall BOOLEAN WIN_FUNC(KeDeregisterBugCheckReasonCallback,1)
+	(void *callback_record)
+{
+	TRACE1("callback_record: %p", callback_record);
+	TODO();
+	return TRUE;
+}
+
 wstdcall void *WIN_FUNC(MmAllocateContiguousMemorySpecifyCache,5)
 	(SIZE_T size, PHYSICAL_ADDRESS lowest, PHYSICAL_ADDRESS highest,
 	 PHYSICAL_ADDRESS boundary, enum memory_caching_type cache_type)
@@ -1687,11 +1691,11 @@ wstdcall void *WIN_FUNC(MmAllocateContiguousMemorySpecifyCache,5)
 	void *addr;
 	gfp_t flags;
 
-	ENTER2("%lu, 0x%lx, 0x%lx, 0x%lx, %d", size, (long)lowest,
-	       (long)highest, (long)boundary, cache_type);
+	ENTER2("%zu, 0x%llx, 0x%llx, 0x%llx, %d", size, lowest,
+	       highest, boundary, cache_type);
 	flags = irql_gfp();
 	addr = wrap_get_free_pages(flags, size);
-	TRACE2("%p, %lu, 0x%x", addr, size, flags);
+	TRACE2("%p, %zu, 0x%x", addr, size, flags);
 	if (addr && ((virt_to_phys(addr) + size) <= highest))
 		EXIT2(return addr);
 #ifdef CONFIG_X86_64
@@ -1707,15 +1711,17 @@ wstdcall void *WIN_FUNC(MmAllocateContiguousMemorySpecifyCache,5)
 	else if (highest > DMA_BIT_MASK(30))
 		flags |= __GFP_HIGHMEM;
 #endif
+	if (addr)
+		free_pages((unsigned long)addr, get_order(size));
 	addr = wrap_get_free_pages(flags, size);
-	TRACE2("%p, %lu, 0x%x", addr, size, flags);
+	TRACE2("%p, %zu, 0x%x", addr, size, flags);
 	return addr;
 }
 
 wstdcall void WIN_FUNC(MmFreeContiguousMemorySpecifyCache,3)
 	(void *base, SIZE_T size, enum memory_caching_type cache_type)
 {
-	TRACE2("%p, %lu", base, size);
+	TRACE2("%p, %zu", base, size);
 	free_pages((unsigned long)base, get_order(size));
 }
 
@@ -1744,14 +1750,14 @@ wstdcall void __iomem *WIN_FUNC(MmMapIoSpace,3)
 		virt = ioremap(phys_addr, size);
 	else
 		virt = ioremap_nocache(phys_addr, size);
-	TRACE1("%Lx, %lu, %p", phys_addr, size, virt);
+	TRACE1("%llx, %zu, %p", phys_addr, size, virt);
 	return virt;
 }
 
 wstdcall void WIN_FUNC(MmUnmapIoSpace,2)
 	(void __iomem *addr, SIZE_T size)
 {
-	ENTER1("%p, %lu", addr, size);
+	ENTER1("%p, %zu", addr, size);
 	iounmap(addr);
 	return;
 }
@@ -1942,7 +1948,7 @@ wstdcall NTSTATUS WIN_FUNC(ObReferenceObjectByHandle,6)
 
 /* DDK doesn't say if return value should be before incrementing or
  * after incrementing reference count, but according to #reactos
- * devels, it should be return value after incrementing */
+ * developers, it should be return value after incrementing */
 wfastcall LONG WIN_FUNC(ObfReferenceObject,1)
 	(void *object)
 {
@@ -2148,7 +2154,7 @@ wstdcall NTSTATUS WIN_FUNC(ZwWriteFile,9)
 	else
 		offset = fo->current_byte_offset;
 	if (length + offset > file->size) {
-		WARNING("%lu, %u", length + offset, (unsigned int)file->size);
+		WARNING("%lu, %zu", length + offset, file->size);
 		/* TODO: implement writing past end of current size */
 		iosb->status = STATUS_FAILURE;
 		iosb->info = 0;
@@ -2256,8 +2262,9 @@ wstdcall NTSTATUS WIN_FUNC(ZwOpenSection,3)
 
 wstdcall NTSTATUS WIN_FUNC(ZwMapViewOfSection,10)
 	(void *secn_handle, void *process_handle, void **base_address,
-	 ULONG zero_bits, LARGE_INTEGER *secn_offset, SIZE_T *view_size,
-	 enum section_inherit inherit, ULONG alloc_type, ULONG protect)
+	 ULONG zero_bits, SIZE_T commit_size, LARGE_INTEGER *secn_offset,
+	 SIZE_T *view_size, enum section_inherit inherit, ULONG alloc_type,
+	 ULONG protect)
 {
 	INFO("%p, %p, %p", secn_handle, process_handle, base_address);
 	TODO();
@@ -2337,7 +2344,7 @@ wstdcall NTSTATUS WIN_FUNC(ZwDeleteKey,1)
 	return STATUS_SUCCESS;
 }
 
-wstdcall NTSTATUS WIN_FUNC(ZwPowerInformation,4)
+wstdcall NTSTATUS WIN_FUNC(ZwPowerInformation,5)
 	(INT info_level, void *in_buf, ULONG in_buf_len, void *out_buf,
 	 ULONG out_buf_len)
 {
@@ -2379,10 +2386,11 @@ wstdcall NTSTATUS WIN_FUNC(WmiQueryTraceInformation,4)
 }
 
 /* this function can't be wstdcall as it takes variable number of args */
+__attribute__((format(printf, 1, 2)))
 noregparm ULONG WIN_FUNC(DbgPrint,12)
 	(char *format, ...)
 {
-#ifdef DEBUG
+#if DEBUG >= 1
 	va_list args;
 	static char buf[100];
 
@@ -2394,10 +2402,29 @@ noregparm ULONG WIN_FUNC(DbgPrint,12)
 	return STATUS_SUCCESS;
 }
 
+__attribute__((format(printf, 3, 4)))
+noregparm ULONG WIN_FUNC(DbgPrintEx,12)
+	(ULONG component_id, ULONG severity, char *format, ...)
+{
+#if DEBUG >= 1
+	va_list args;
+	static char buf[100];
+
+	va_start(args, format);
+	vsnprintf(buf, sizeof(buf), format, args);
+	TRACE1("component_id: %d, severity: %d\n", component_id, severity);
+	printk(KERN_DEBUG "%s (%s): %s", DRIVER_NAME, __func__, buf);
+	va_end(args);
+#endif
+	return STATUS_SUCCESS;
+}
+
 wstdcall void WIN_FUNC(KeBugCheck,1)
 	(ULONG code)
 {
-	TODO();
+	ERROR("Unrecoverable error reported by the driver");
+	ERROR("code: 0x%x\n", code);
+	dump_stack();
 	return;
 }
 
@@ -2405,7 +2432,10 @@ wstdcall void WIN_FUNC(KeBugCheckEx,5)
 	(ULONG code, ULONG_PTR param1, ULONG_PTR param2,
 	 ULONG_PTR param3, ULONG_PTR param4)
 {
-	TODO();
+	ERROR("Unrecoverable error reported by the driver");
+	ERROR("code: 0x%x, params: 0x%lx 0x%lx 0x%lx 0x%lx\n", code, param1,
+	      param2, param3, param4);
+	dump_stack();
 	return;
 }
 
@@ -2447,49 +2477,12 @@ wstdcall void WIN_FUNC(_purecall,0)
 	TODO();
 }
 
-wstdcall void WIN_FUNC(__chkstk,0)
-	(void)
-{
-	TODO();
-}
-
 struct worker_init_struct {
-	work_struct_t work;
+	struct work_struct work;
 	struct completion completion;
 	struct nt_thread *nt_thread;
 };
 
-static void wrap_worker_init_func(worker_param_t param)
-{
-	struct worker_init_struct *worker_init_struct;
-
-	worker_init_struct =
-		worker_param_data(param, struct worker_init_struct, work);
-	TRACE1("%p", worker_init_struct);
-	worker_init_struct->nt_thread = create_nt_thread(current);
-	if (!worker_init_struct->nt_thread)
-		WARNING("couldn't create worker thread");
-	complete(&worker_init_struct->completion);
-}
-
-struct nt_thread *wrap_worker_init(workqueue_struct_t *wq)
-{
-	struct worker_init_struct worker_init_struct;
-
-	TRACE1("%p", &worker_init_struct);
-	init_completion(&worker_init_struct.completion);
-	initialize_work(&worker_init_struct.work, wrap_worker_init_func,
-			&worker_init_struct);
-	worker_init_struct.nt_thread = NULL;
-	if (wq)
-		queue_work(wq, &worker_init_struct.work);
-	else
-		schedule_work(&worker_init_struct.work);
-	wait_for_completion(&worker_init_struct.completion);
-	TRACE1("%p", worker_init_struct.nt_thread);
-	return worker_init_struct.nt_thread;
-}
-
 int ntoskernel_init(void)
 {
 	struct timeval now;
@@ -2508,8 +2501,8 @@ int ntoskernel_init(void)
 
 	nt_spin_lock_init(&nt_list_lock);
 
-	initialize_work(&kdpc_work, kdpc_worker, NULL);
-	initialize_work(&ntos_work, ntos_work_worker, NULL);
+	INIT_WORK(&kdpc_work, kdpc_worker);
+	INIT_WORK(&ntos_work, ntos_work_worker);
 	wrap_timer_slist.next = NULL;
 
 	do_gettimeofday(&now);
@@ -2517,17 +2510,22 @@ int ntoskernel_init(void)
 	wrap_ticks_to_boot += (u64)now.tv_sec * TICKSPERSEC;
 	wrap_ticks_to_boot += now.tv_usec * 10;
 	wrap_ticks_to_boot -= jiffies * TICKSPERJIFFY;
-	TRACE2("%Lu", wrap_ticks_to_boot);
+	TRACE2("%llu", wrap_ticks_to_boot);
+
+	cpu_count = num_online_cpus();
 
 #ifdef WRAP_PREEMPT
 	do {
 		int cpu;
 		for_each_possible_cpu(cpu) {
-			irql_info_t *info;
+			struct irql_info *info;
 			info = &per_cpu(irql_info, cpu);
 			mutex_init(&(info->lock));
 			info->task = NULL;
 			info->count = 0;
+#ifdef CONFIG_SMP
+			cpumask_setall(&info->cpus_allowed);
+#endif
 		}
 	} while (0);
 #endif
@@ -2537,8 +2535,7 @@ int ntoskernel_init(void)
 		WARNING("couldn't create ntos_wq thread");
 		return -ENOMEM;
 	}
-	ntos_worker_thread = wrap_worker_init(ntos_wq);
-	TRACE1("%p", ntos_worker_thread);
+	TRACE1("ntos_wq: %p", ntos_wq);
 
 	if (add_bus_driver("PCI")
 #ifdef ENABLE_USB
@@ -2549,7 +2546,7 @@ int ntoskernel_init(void)
 		return -ENOMEM;
 	}
 	mdl_cache =
-		wrap_kmem_cache_create("wrap_mdl",
+		wrap_kmem_cache_create(DRIVER_NAME "_mdl",
 				       sizeof(struct wrap_mdl) + MDL_CACHE_SIZE,
 				       0, 0);
 	TRACE2("%p", mdl_cache);
@@ -2564,7 +2561,7 @@ int ntoskernel_init(void)
 	*((ULONG64 *)&kuser_shared_data.system_time) = ticks_1601();
 	init_timer(&shared_data_timer);
 	shared_data_timer.function = update_user_shared_data_proc;
-	shared_data_timer.data = (unsigned long)0;
+	shared_data_timer.data = 0;
 #endif
 	return 0;
 }
@@ -2661,9 +2658,6 @@ void ntoskernel_exit(void)
 #endif
 	if (ntos_wq)
 		destroy_workqueue(ntos_wq);
-	TRACE1("%p", ntos_worker_thread);
-	if (ntos_worker_thread)
-		ObDereferenceObject(ntos_worker_thread);
 	ENTER2("freeing objects");
 	spin_lock_bh(&ntoskernel_lock);
 	while ((cur = RemoveHeadList(&object_list))) {
