@@ -51,6 +51,20 @@
 #error "this module is for x86 or x86_64 architectures only"
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,14)
+#define gfp_t unsigned int __nocast
+
+static inline void *_kzalloc(size_t size, gfp_t flags)
+{
+	void *p = kmalloc(size, flags);
+	if (likely(p != NULL))
+		memset(p, 0, size);
+	return p;
+}
+
+#define kzalloc(size, flags) _kzalloc(size, flags)
+#endif
+
 /* Interrupt backwards compatibility stuff */
 #include <linux/interrupt.h>
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,29)
@@ -76,6 +90,7 @@
 #define set_cpus_allowed_ptr(task, mask) set_cpus_allowed(task, *mask)
 #endif /* Linux < 2.6.26 */
 
+#ifdef CONFIG_SMP
 #if LINUX_VERSION_CODE < KERNEL_VERSION(2,6,28)
 #define cpumask_copy(dst, src) do { *dst = *src; } while (0)
 #define cpumask_equal(mask1, mask2) cpus_equal(*mask1, *mask2)
@@ -87,6 +102,7 @@ static cpumask_t cpumasks[NR_CPUS];
 	&cpumasks[cpu];				\
 })
 #endif /* Linux < 2.6.28 */
+#endif /* CONFIG_SMP */
 
 #ifndef tsk_cpus_allowed
 #define tsk_cpus_allowed(tsk) (&(tsk)->cpus_allowed)
@@ -239,6 +255,10 @@ static inline void (INIT_WORK)(struct work_struct *work, work_func_t func)
 #define IRQF_SHARED SA_SHIRQ
 #endif
 
+#ifndef UMH_WAIT_PROC
+#define UMH_WAIT_PROC 1
+#endif
+
 #define memcpy_skb(skb, from, length)			\
 	memcpy(skb_put(skb, length), from, length)
 
@@ -266,6 +286,14 @@ static inline void (INIT_WORK)(struct work_struct *work, work_func_t func)
 #define usb_free_coherent(dev, size, addr, dma) (usb_buffer_free((dev), (size), (addr), (dma)))
 #endif
 
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(3,8,0)
+#define daemonize(name, ...) do {} while (0)
+#endif
+
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3,9,0)
+#define add_taint(flag, lockdep_ok) add_taint(flag)
+#endif
+
 #include "winnt_types.h"
 #include "ndiswrapper.h"
 #include "pe_linker.h"
@@ -315,6 +343,17 @@ static inline void netif_poll_disable(struct net_device *dev)
 #endif
 #endif
 
+#if LINUX_VERSION_CODE < KERNEL_VERSION(3,11,0)
+#define netdev_notifier_info_to_dev(x) ((struct net_device *)(x))
+#endif
+
+#ifdef INIT_COMPLETION
+static inline void reinit_completion(struct completion *x)
+{
+	INIT_COMPLETION(*x);
+}
+#endif
+
 /* TICK is 100ns */
 #define TICKSPERSEC		10000000
 #define TICKSPERMSEC		10000
