@@ -1,5 +1,6 @@
 /*
  *  Copyright (C) 2003-2005 Pontus Fuchs, Giridhar Pemmasani
+ *  Copyright (C) 2006-2007 Giridhar Pemmasani
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -22,15 +23,16 @@ wstdcall SIZE_T WIN_FUNC(RtlCompareMemory,3)
 	size_t i;
 	char *x, *y;
 
+	ENTER1("%p %p %zd", a, b, len);
 	x = (char *)a;
 	y = (char *)b;
 	/* MSDN says this should return number of bytes that compare as
-	 * equal. This can be interpretted as either all bytes that are
+	 * equal. This can be interpreted as either all bytes that are
 	 * equal in 'len' bytes or that only until the bytes compare as
 	 * not equal. Initially we had it the former way, but Realtek driver
 	 * doesn't like it that way - it takes many attempts to associate
 	 * with WPA. ReactOS returns the number of bytes that are equal
-	 * upto when they compare as not equal.
+	 * before the first differing byte.
 	 * According to lords at #reactos, that is the way it should be
 	 * and that msdn is wrong about it!
 	 */
@@ -202,7 +204,7 @@ wstdcall ULONG WIN_FUNC(RtlxAnsiStringToUnicodeSize,1)
 
 	for (i = 0; i < string->max_length && string->buf[i]; i++)
 		;
-	return (i * sizeof(wchar_t));
+	return i * sizeof(wchar_t);
 }
 
 wstdcall ULONG WIN_FUNC(RtlxUnicodeStringToAnsiSize,1)
@@ -451,9 +453,9 @@ wstdcall NTSTATUS WIN_FUNC(RtlUpcaseUnicodeString,3)
 
 	if (alloc) {
 		dst->buf = ExAllocatePoolWithTag(NonPagedPool, src->length, 0);
-		if (dst->buf) {
+		if (dst->buf)
 			dst->max_length = src->length;
-		} else
+		else
 			EXIT2(return STATUS_NO_MEMORY);
 	} else {
 		if (dst->max_length < src->length)
@@ -704,13 +706,10 @@ wstdcall void WIN_FUNC(RtlRaiseException,1)
 	TODO();
 }
 
-int rtl_init(void)
+wstdcall BOOLEAN WIN_FUNC(RtlIsServicePackVersionInstalled,1)
+	(ULONG version)
 {
-	return 0;
-}
-
-/* called when module is being removed */
-void rtl_exit(void)
-{
-	EXIT4(return);
+	/* Assume we have all service packs */
+	TRACE1("version: %d", version);
+	return TRUE;
 }
