@@ -40,13 +40,12 @@
 
 #define SETTING_LEN (MAX_SETTING_NAME_LEN + MAX_SETTING_VALUE_LEN + 2)
 
-static const char confdir[] = "/etc/ndiswrapper";
-static const char ioctl_file[] = "/dev/ndiswrapper";
+static const char confdir[] = DRIVER_CONFIG_DIR;
+static const char ioctl_file[] = "/dev/" DRIVER_NAME;
 static int debug;
 
 #ifndef UTILS_VERSION
-#error compile this file with 'make' in the 'utils' \
-	directory only
+#error "compile this file with 'make' in the 'utils' directory only"
 #endif
 
 #define LOG_MSG(where, fmt, ...)					\
@@ -83,7 +82,7 @@ static int load_file(char *filename, struct load_driver_file *driver_file)
 	}
 	size = statbuf.st_size;
 
-	image = mmap(0, size, PROT_READ, MAP_PRIVATE, fd, 0);
+	image = mmap(NULL, size, PROT_READ, MAP_PRIVATE, fd, 0);
 	if (image == MAP_FAILED) {
 		ERROR("unable to mmap driver: %s", strerror(errno));
 		close(fd);
@@ -105,15 +104,17 @@ static int parse_setting_line(const char *setting_line, char *setting_name,
 	char *val, *end;
 	int i;
 
-	// We try to be really paranoid parsing settings
+	/* We try to be really paranoid parsing settings */
 	for (s = setting_line; isspace(*s); s++)
 		;
 
-	// ignore comments and blank lines
+	/* ignore comments and blank lines */
 	if (*s == '#' || *s == ';' || *s == '\0')
 		return 0;
-	if ((val = strchr(s, '|')) == NULL ||
-	    (end = strchr(s, '\n')) == NULL) {
+
+	val = strchr(s, '|');
+	end = strchr(s, '\n');
+	if (val == NULL || end == NULL) {
 		ERROR("invalid setting: %s", setting_line);
 		return -EINVAL;
 	}
@@ -134,7 +135,7 @@ static int parse_setting_line(const char *setting_line, char *setting_name,
 	}
 	DBG("Found setting: name=%s, val=\"%s\"", setting_name, setting_val);
 
-	// setting_val can be empty, but not name
+	/* setting_val can be empty, but not name */
 	if (strlen(setting_name) == 0) {
 		ERROR("invalid setting: \"%s\"", setting_line);
 		return -EINVAL;
@@ -175,7 +176,8 @@ static int read_conf_file(char *conf_file_name, struct load_driver *driver)
 	num_settings = 0;
 	driver->num_settings = 0;
 
-	if ((config = fopen(conf_file_name, "r")) == NULL) {
+	config = fopen(conf_file_name, "r");
+	if (config == NULL) {
 		ERROR("unable to open config file: %s", strerror(errno));
 		return -EINVAL;
 	}
@@ -240,7 +242,8 @@ static int load_bin_file(int ioctl_device, char *driver_name, char *file_name)
  * open a windows driver and pass it to the kernel module.
  * returns 0: on success, -1 on error
  */
-static int load_driver(int ioctl_device, char *driver_name, char *conf_file_name)
+static int load_driver(int ioctl_device, char *driver_name,
+		       char *conf_file_name)
 {
 	int i;
 	struct dirent *dirent;
@@ -258,13 +261,15 @@ static int load_driver(int ioctl_device, char *driver_name, char *conf_file_name
 		      driver_name, strerror(errno));
 		return -EINVAL;
 	}
-	if ((driver_dir = opendir(".")) == NULL) {
+	driver_dir = opendir(".");
+	if (driver_dir == NULL) {
 		ERROR("couldn't open driver directory %s: %s",
 		      driver_name, strerror(errno));
 		return -EINVAL;
 	}
 
-	if ((driver = malloc(sizeof(*driver))) == NULL) {
+	driver = malloc(sizeof(*driver));
+	if (driver == NULL) {
 		ERROR("couldn't allocate memory for driver %s", driver_name);
 		goto err;
 	}
@@ -299,7 +304,8 @@ static int load_driver(int ioctl_device, char *driver_name, char *conf_file_name
 		     strcasecmp(&dirent->d_name[len-5], ".conf") == 0)
 			continue;
 
-		if (len > 4 && strcasecmp(&dirent->d_name[len-4], ".sys") == 0) {
+		if (len > 4 &&
+		    strcasecmp(&dirent->d_name[len-4], ".sys") == 0) {
 			if (load_file(dirent->d_name,
 				      &driver->sys_files[num_sys_files])) {
 				ERROR("couldn't load .sys file %s",
@@ -333,7 +339,7 @@ static int load_driver(int ioctl_device, char *driver_name, char *conf_file_name
 	}
 
 	if (num_sys_files == 0) {
-		ERROR("coudln't find valid drivers files for driver %s",
+		ERROR("couldn't find valid drivers files for driver %s",
 		      driver_name);
 		goto err;
 	}
@@ -396,7 +402,10 @@ static int get_device(char *driver_name, int vendor, int device, int subvendor,
 		ld->subdevice = 0;
 		ret = 0;
 	}
-	chdir("..");
+	if (chdir(confdir)) {
+		ERROR("couldn't chdir to %s: %s", confdir, strerror(errno));
+		return -EINVAL;
+	}
 	if (ret)
 		ld->vendor = 0;
 	else {
@@ -426,7 +435,8 @@ static int load_device(int ioctl_device, int vendor, int device,
 		ERROR("couldn't chdir to %s: %s", confdir, strerror(errno));
 		return -EINVAL;
 	}
-	if ((dir = opendir(".")) == NULL) {
+	dir = opendir(".");
+	if (dir == NULL) {
 		ERROR("directory %s is not valid: %s",
 		      confdir, strerror(errno));
 		return -EINVAL;
@@ -457,7 +467,7 @@ static int load_device(int ioctl_device, int vendor, int device,
   * we create a device in /dev instead of /tmp as some distributions don't
   * allow creation of devices in /tmp
   */
-static int get_ioctl_device()
+static int get_ioctl_device(void)
 {
 	int fd, minor_dev;
 	char line[64];
@@ -505,7 +515,8 @@ static int get_ioctl_device()
 
 int main(int argc, char *argv[0])
 {
-	int i, ioctl_device, res;
+	int i, res;
+	int ioctl_device = -1;
 	char *cmd;
 
 	openlog(PROG_NAME, LOG_PERROR | LOG_CONS, LOG_KERN | LOG_DEBUG);
@@ -583,6 +594,10 @@ int main(int argc, char *argv[0])
 			goto out;
 		}
 		res = load_bin_file(ioctl_device, argv[4], argv[5]);
+	} else {
+		ERROR("incorrect usage of %s (%d)", argv[0], argc);
+		res = 13;
+		goto out;
 	}
 out:
 	if (ioctl_device != -1)
@@ -590,4 +605,3 @@ out:
 	closelog();
 	return res;
 }
-
